-- this is the heal/treats/etc buttons along topright of frame/journal (and bottom of minipanel)

local _,L = ...
local rematch = Rematch
local toolbar = RematchToolbar

local blizzBugFixLoginTime

rematch:InitModule(function()
	rematch.Toolbar = toolbar
	toolbar.FindBattleButton.tooltipTitle = FIND_BATTLE
	toolbar.FindBattleButton.tooltipBody = BATTLE_PETS_FIND_BATTLE_TOOLTIP

	-- WoW patch 6.2 has a bug where logging out/in will not show proper cooldowns spinners on existing buffs
	-- with SetCooldown. To see this in action: hearthstone, wait a few mins, log out and log back in and look
	-- at its cooldown. It will be at the "just cast" point of animation and stay there until CD is up.
	-- To get around this, if either pet treat is up upon login, then blizzBugFixLoginTime will be the GetTime()
	-- of this login, and any cooldowns that go before this time will be adjusted to start at this time.
	-- (for Legion: test if this is still necessary; hopefully blizzard fixes this!)
	C_Timer.After(0.1,function()
		if UnitBuff("player",GetItemSpell("item:98114") or "") or UnitBuff("player",GetItemSpell("item:98112") or "") then
			blizzBugFixLoginTime = GetTime()
		end
	end)

end)

function toolbar:OnShow()
	self:RegisterEvent("COMPANION_UPDATE")
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterUnitEvent("UNIT_AURA","player")
	self:RegisterUnitEvent("UNIT_SPELLCAST_SUCCEEDED","player")
end

function toolbar:OnHide()
	self:UnregisterEvent("COMPANION_UPDATE")
	self:UnregisterEvent("BAG_UPDATE")
	self:UnregisterEvent("UNIT_AURA")
	self:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
end

-- the toolbar has a separate event handler; all events are only registered while the toolbar is visible
function toolbar:OnEvent(event,...)
	local update
	if event=="UNIT_AURA" or event=="BAG_UPDATE" then -- can't qualify what changed on these two
		update = true -- possibly bandages, treats or safari hat added/removed from inventory
	elseif event=="UNIT_SPELLCAST_SUCCEEDED" and select(5,...)==125439 then
		update = true -- revive battle pets cast
	elseif event=="COMPANION_UPDATE" and select(1,...)=="CRITTER" then
		update = true -- summoned pet changed
	end
	-- no toolbar update is critical, delaying the update to 0.1 seconds after last noteworthy event firing
	if update then
		rematch:StartTimer("Toolbar",0.1,toolbar.Update)
	end
end

function toolbar:Reparent(parent)
	if toolbar:GetParent()~=parent then
		toolbar:SetParent(parent)
		toolbar:ClearAllPoints()
		local narrow = rematch.Frame.TitleBar.NarrowModeButton
		if parent==rematch.Frame then -- parenting to frame for minimized view
			toolbar:SetWidth(251)
			toolbar:SetPoint("BOTTOMLEFT",7,4)
			narrow:Hide()
		elseif parent==rematch.NarrowPanel then -- narrow mode
			-- in narrow mode toolbar, make bar match the width of minimized so buttons stay
--			if RematchSettings.CornerPos=="BOTTOMLEFT" or RematchSettings.CornerPos=="TOPLEFT" then
--				toolbar:SetWidth(251)
--			else
				toolbar:SetWidth(246)
--			end
			toolbar:SetPoint("BOTTOMLEFT",2,0)
			narrow:Show()
		elseif parent==rematch.TopPanel then -- parenting to TopPanel (for expanded frame and journal)
			toolbar:SetWidth(244)
			toolbar:SetPoint("RIGHT",-6,0)
			narrow:Show()
		end
		toolbar:ReorderButtons()
		toolbar:Show()
	end
end

function toolbar:Update()
	if not toolbar:IsVisible() then return end
	toolbar.HealButton.Cooldown:SetCooldown(GetSpellCooldown(125439))
	toolbar:UpdateItemButton(toolbar.BandageButton,86143)
	toolbar:UpdateItemButton(toolbar.PetTreatButton,98114,true)
	toolbar:UpdateItemButton(toolbar.LesserTreatButton,98112,true)
	toolbar:DimButton(toolbar.SafariButton,not PlayerHasToy(92738))

	local petID = C_PetJournal.GetSummonedPetGUID()
	if petID then
		local _,customName,_,_,_,_,_,name,icon = C_PetJournal.GetPetInfoByPetID(petID)
		toolbar.SummonButton.Icon:SetTexture(icon)
		toolbar.SummonButton.tooltipTitle = L["Dismiss Pet"]
		toolbar.SummonButton.tooltipBody = customName or name
		toolbar.SummonButton.Cancel:Show()
		toolbar.SummonButton:SetAttribute("macrotext","/dismisspet")
		toolbar.SummonButton:SetAttribute("macrotext2","/dismisspet")
	else
		toolbar.SummonButton.Icon:SetTexture("Interface\\Icons\\Achievement_GuildPerk_MountUp")
		toolbar.SummonButton.tooltipTitle = L["Summon Pet"]
		if RematchSettings.ToolbarDismiss then
			toolbar.SummonButton.tooltipBody = L["Summons a random pet from your favorites."]
		else
			toolbar.SummonButton.tooltipBody = format("%s %s\n%s %s",rematch.LMB,L["Random Favorite"],rematch.RMB,L["Random From All"])
		end
		toolbar.SummonButton.Cancel:Hide()
		toolbar.SummonButton:SetAttribute("macrotext","/run C_PetJournal.SummonRandomPet(false)")
		toolbar.SummonButton:SetAttribute("macrotext2","/run C_PetJournal.SummonRandomPet(true)")
	end

	local queueState = C_PetBattles.GetPVPMatchmakingInfo();
	if queueState=="queued" or queueState=="proposal" or queueState=="suspended" then
		toolbar.FindBattleButton.Icon:SetTexture("Interface\\Icons\\PetBattle_Attack-Down")
		if queueState=="proposal" then
			toolbar.FindBattleButton:SetAttribute("macrotext","/run C_PetBattles.DeclineQueuedPVPMatch()")
		else
			toolbar.FindBattleButton:SetAttribute("macrotext","/run C_PetBattles.StopPVPMatchmaking()")
		end
	else
		toolbar.FindBattleButton.Icon:SetTexture("Interface\\Icons\\PetBattle_Attack")
		toolbar.FindBattleButton:SetAttribute("macrotext","/run C_PetBattles.StartPVPMatchmaking()")
	end

	local safariBuff = GetItemSpell(92738)
	if safariBuff and UnitBuff("player",safariBuff) then
		toolbar.SafariButton.Cancel:Show()
		toolbar.SafariButton:SetAttribute("type","macro")
		toolbar.SafariButton:SetAttribute("macrotext",format("/cancelaura %s",safariBuff))
	else
		toolbar.SafariButton.Cancel:Hide()
		toolbar.SafariButton:SetAttribute("type","item")
		toolbar.SafariButton:SetAttribute("item",(GetItemInfo(92738)))
	end
end

function toolbar:UpdateItemButton(button,itemID,showTimeLeft)
	local count = GetItemCount(itemID)
	button.Count:SetText(count)
	toolbar:DimButton(button,count==0)
	if showTimeLeft then

		local spell = GetItemSpell(itemID)
		if spell then
			local _,_,_,_,_,duration,expiration = UnitBuff("player",spell)
			if duration then
				local start = expiration - duration
				if blizzBugFixLoginTime and start<blizzBugFixLoginTime then
					-- this cooldown was from a previous session, adjust times to login time
					-- this means if a treat is at 30 mins remaining, the spinner's duration will be 30 mins with a start of the login time
					button.Cooldown:SetCooldown(blizzBugFixLoginTime,expiration-blizzBugFixLoginTime)
				else
					button.Cooldown:SetCooldown(expiration-duration,duration)
				end
			else
				button.Cooldown:Hide()
			end
		end

	end
end

function toolbar:DimButton(button,dim,desaturate)
	button.Icon:SetDesaturated(desaturate and true or false)
	if dim then
		button.Icon:SetVertexColor(0.5,0.5,0.5)
	else
		button.Icon:SetVertexColor(1,1,1)
	end
end

-- this is the OnEnter for all of the toolbar buttons
function toolbar:ButtonOnEnter(once)
	if self.tooltipTitle then
		rematch.ShowTooltip(self)
	else
		GameTooltip:SetOwner(self,"ANCHOR_NONE") -- item or spells will use GameTooltip
		local itemID = self:GetAttribute("item")
		local spellID = self:GetAttribute("spell")
		if itemID==GetItemInfo(92738) then
			itemID = "item:92738" -- can't use item:id attributes for toybox, Safari Hat is named attribute
		end
		if itemID then
			local buff = GetItemSpell(itemID)
			local index, found = 1
			-- using this old buff-lookup method in the OnEnter because we need a buff index for SetUnitBuff
			repeat
				local candidate = UnitBuff("player",index)
				if candidate == buff then
					found = index
				end
				index = index + 1
			until not candidate or found
			if found then
				GameTooltip:SetUnitBuff("player",found) -- display buff tooltip if it's up
			elseif itemID=="item:92738" then
				GameTooltip:SetToyByItemID(92738) -- safari hat toybox tooltip
			else
				GameTooltip:SetHyperlink(itemID) -- rest of items
			end
		elseif spellID then
			GameTooltip:SetSpellByID(tonumber(spellID))
		end
		rematch:SmartAnchor(GameTooltip,self)
		GameTooltip:SetBackdropBorderColor(0.5,0.5,0.5)
		GameTooltip:Show()
	end

	-- this timer calls this onenter again in half a second; the onleave stops the timer
	-- this causes a bit of garbage creation but is forgivable for the effects (cooldown timers
	-- count down in the tooltip and left/right click instructions update)
	if not once then
		self.tooltipTimer = C_Timer.NewTimer(0.5,function() toolbar.ButtonOnEnter(self) end)
	end
end

function toolbar:ButtonOnLeave()
	if self.tooltipTimer then
		self.tooltipTimer:Cancel() -- stop tooltip updating itself
	end
	if self.tooltipTitle then
		rematch:HideTooltip() -- if it had a tooltipTitle, it's a rematch tooltip
	else
		GameTooltip:Hide() -- otherwise a default tooltip
	end
end

-- all toolbar buttons use this PostClick
function toolbar:ButtonPostClick(button)
	-- for heal, bandage and treat buttons that are turned off in their pre-click
	if self.backupAttribute then
		self:SetAttribute("type",self.backupAttribute)
		self.backupAttribute = nil
	end
	-- if ToolbarDismiss enabled and right-click used, hide rematch
	if button=="RightButton" and RematchSettings.ToolbarDismiss then
		if rematch.Frame:IsVisible() then
			rematch.Frame.Toggle()
		elseif rematch.Journal:IsVisible() or (PetJournal and PetJournal:IsVisible()) then
			HideUIPanel(CollectionsJournal)
		end
	end
end

-- only heal buttons get a PreClick
function toolbar:HealButtonPreClick()
	if InCombatLockdown() then return end
	local roster = rematch.Roster
	roster:CheckForUpdate()
	for i=1,roster:GetNumPets(true) do
		local petID = roster:GetPetByIndex(i,true)
		if type(petID)=="string" then
			local health,maxHealth = C_PetJournal.GetPetStats(petID)
			if health<maxHealth then
				return
			end
		end
	end
	-- if we reached here, all pets are healed
	self.backupAttribute = self:GetAttribute("type")
	self:SetAttribute("type",nil)
	if toolbar:IsVisible() then -- if toolbar up, notify via tooltip
		if self.tooltipTimer then self.tooltipTimer:Cancel() end
		toolbar.ButtonOnEnter(self,true)
		GameTooltip:AddLine(format("%s%s",rematch.hexBlue,L["All pets are at full health."]))
		GameTooltip:Show()
	else -- if toolbar not visible, notify via print
		rematch:print(format("%s%s",rematch.hexBlue,L["All pets are at full health."]))
	end
end

function toolbar:TreatButtonPreClick()
	if InCombatLockdown() then return end
	local buff = GetItemSpell(self:GetAttribute("item"))
	if buff and UnitBuff("player",buff) then -- buff is already up
		-- while treats CAN be recast, it doesn't refresh duration of buff, so we should prevent their use
		self.backupAttribute = self:GetAttribute("type")
		self:SetAttribute("type",nil)
		if toolbar:IsVisible() then
			if self.tooltipTimer then self.tooltipTimer:Cancel() end
			toolbar.ButtonOnEnter(self,true)
			GameTooltip:AddLine(format("%s%s",rematch.hexBlue,L["This treat's buff is already active."]),nil,nil,nil,true)
			GameTooltip:Show()
		end
	end
end

-- depending on settings.ReverseToolbar, change the displayed order of toolbar buttons
function toolbar:ReorderButtons()
	for k,v in pairs({"HealButton","BandageButton","PetTreatButton","LesserTreatButton","SafariButton","SummonButton","FindBattleButton"}) do
		toolbar[v]:ClearAllPoints()
	end
	if RematchSettings.ReverseToolbar then
		toolbar.HealButton:SetPoint("LEFT",-4,0)
		toolbar.BandageButton:SetPoint("LEFT",28,0)
		toolbar.PetTreatButton:SetPoint("CENTER",32,0)
		toolbar.LesserTreatButton:SetPoint("CENTER")
		toolbar.SafariButton:SetPoint("CENTER",-32,0)
		toolbar.SummonButton:SetPoint("RIGHT",-27,0)
		toolbar.FindBattleButton:SetPoint("RIGHT",5,0)
	else
		toolbar.HealButton:SetPoint("RIGHT",5,0)
		toolbar.BandageButton:SetPoint("RIGHT",-27,0)
		toolbar.PetTreatButton:SetPoint("CENTER",-32,0)
		toolbar.LesserTreatButton:SetPoint("CENTER")
		toolbar.SafariButton:SetPoint("CENTER",32,0)
		toolbar.SummonButton:SetPoint("LEFT",28,0)
		toolbar.FindBattleButton:SetPoint("LEFT",-4,0)
	end
end

